#if defined( HAVE_CONFIG_H )
# include "config.h"
#endif

#include <tdemacros.h>
#include <gmodule.h>
#include <stdio.h>
#include "qt_rc_style.h"
#include "qt_style.h"

#include "qt_qt_wrapper.h"

TDE_EXPORT void theme_init (GTypeModule *module);
TDE_EXPORT void theme_exit (void);
TDE_EXPORT GtkRcStyle * theme_create_rc_style (void);

TDE_EXPORT void theme_init (GTypeModule *module)
{
	createTQApp();
	qtengine_rc_style_register_type (module);
	qtengine_style_register_type (module);
}

TDE_EXPORT void theme_exit (void)
{
	destroyTQApp();
}

TDE_EXPORT GtkRcStyle * theme_create_rc_style (void)
{
	void *ptr = GTK_RC_STYLE (g_object_new (TQTENGINE_TYPE_RC_STYLE, NULL));  
	return (GtkRcStyle *)ptr;
}
