#include <gtk/gtkstyle.h>

typedef struct _QtEngineStyle QtEngineStyle;
typedef struct _QtEngineStyleClass QtEngineStyleClass;

extern GType qtengine_type_style;

#define TQTENGINE_TYPE_STYLE              qtengine_type_style
#define TQTENGINE_STYLE(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), TQTENGINE_TYPE_STYLE, QtEngineStyle))
#define TQTENGINE_STYLE_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), TQTENGINE_TYPE_STYLE, QtEngineStyleClass))
#define TQTENGINE_IS_STYLE(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), TQTENGINE_TYPE_STYLE))
#define TQTENGINE_IS_STYLE_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), TQTENGINE_TYPE_STYLE))
#define TQTENGINE_STYLE_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), TQTENGINE_TYPE_STYLE, QtEngineStyleClass))

struct _QtEngineStyle
{
	GtkStyle parent_instance;
	
	GdkPixmap* menuBackground;
};

struct _QtEngineStyleClass
{
	GtkStyleClass parent_class;
};

void qtengine_style_register_type (GTypeModule *module);


