#include <gtk/gtkrc.h>

typedef struct _QtEngineRcStyle QtEngineRcStyle;
typedef struct _QtEngineRcStyleClass QtEngineRcStyleClass;

extern GType qtengine_type_rc_style;

#define TQTENGINE_TYPE_RC_STYLE              qtengine_type_rc_style
#define TQTENGINE_RC_STYLE(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), TQTENGINE_TYPE_RC_STYLE, QtEngineRcStyle))
#define TQTENGINE_RC_STYLE_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), TQTENGINE_TYPE_RC_STYLE, QtEngineRcStyleClass))
#define TQTENGINE_IS_RC_STYLE(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), TQTENGINE_TYPE_RC_STYLE))
#define TQTENGINE_IS_RC_STYLE_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), TQTENGINE_TYPE_RC_STYLE))
#define TQTENGINE_RC_STYLE_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), TQTENGINE_TYPE_RC_STYLE, QtEngineRcStyleClass))

struct _QtEngineRcStyle
{
	GtkRcStyle parent_instance;
};

struct _QtEngineRcStyleClass
{
	GtkRcStyleClass parent_class;
};

void qtengine_rc_style_register_type (GTypeModule *module);


